/* $Id: scoring.c,v 1.1.1.2 2004/08/12 15:20:17 roman Exp $ */
/* written by Roman R. Stocsits, 2004 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "utils.h"
#include "structs.h"
#include "blosum62.h"

extern int naSCORE[5][5];
extern int gapOPEN;
extern int gapEXT;
extern struct SEQdata *SEQ;
extern struct ALNdata *aln;
extern struct Factors factors;

long MscoreM(struct ALNdata tempaln[2], int c, int g, int h, short use_factors);
long MscoreIx(struct ALNdata tempaln[2], int c, int g, int h);
long MscoreIy(struct ALNdata tempaln[2], int c, int g, int h);

long scoreM(int a, int b, int c, int g, int h, short use_factors);
long scoreIx(int a, int b, int c, int g, int h);
long scoreIy(int a, int b, int c, int g, int h);

/* ----------------------------------------------------------------------- */

long scoreM(int a, int b, int c, int g, int h, short use_factors){
  
  int x=0, y=0, z=0;
  float aa_orf=1, ns_code=1;
  long s=0, as=0, sM, sIx, sIy;

  h=h-1;
  g=g-1;
  
  s = (naSCORE[SEQ[a].letter[h].nuclNo][SEQ[b].letter[g].nuclNo]);
  
  if(use_factors){
    if(SEQ[a].letter[h].nucl == SEQ[b].letter[g].nucl)  s = s * factors.match;
    else   s = s * factors.mismatch;
  }
  
   for(z=0; z<3; z++){
    if((SEQ[a].letter[h].aa[z]) && (SEQ[b].letter[g].aa[z])){
      if((isalpha(SEQ[a].letter[h].aa[z])) && (isalpha(SEQ[b].letter[g].aa[z]))){
	x=0;  y=0;
	if(z==0){ aa_orf = factors.aa_orf1;  ns_code = factors.eins_coding; }
	if(z==1){ aa_orf = factors.aa_orf2;  ns_code = factors.zwei_coding; }
	if(z==2){ aa_orf = factors.aa_orf3;  ns_code = factors.drei_coding; }
	while((x<23) && (SEQ[a].letter[h].aa[z] != AAorder[x]))      x++;
	while((y<23) && (SEQ[b].letter[g].aa[z] != AAorder[y]))      y++;
	if (x>22 || y>22) continue;
	as = as + (((BLOSUM62[y][x] *100) /2)*aa_orf);
	s = s * ns_code;
      }
    }
  }
  if(!(SEQ[a].letter[h].aa[0]) && !(SEQ[b].letter[g].aa[0]) &&
     !(SEQ[a].letter[h].aa[1]) && !(SEQ[b].letter[g].aa[1]) &&
     !(SEQ[a].letter[h].aa[2]) && !(SEQ[b].letter[g].aa[2]) ){
    ns_code = factors.non_code;
    s = s * ns_code;
  }
  s = s + as;
  sM = aln[c].SCmatrix[0][g][h] + s;
  sIx = aln[c].SCmatrix[1][g][h] + s;
  sIy = aln[c].SCmatrix[2][g][h] + s;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[0][g][h]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[0][g][h]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[0][g][h]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

long scoreIx(int a, int b, int c, int g, int h){
  
  long sM, sIx, sIy;
  
  sM = aln[c].SCmatrix[0][g][h-1] + gapOPEN * factors.gap_open;
  sIx = aln[c].SCmatrix[1][g][h-1] + gapEXT * factors.gap_ext;
  sIy = aln[c].SCmatrix[2][g][h-1] + gapOPEN * factors.gap_open;
  /* printf("%i  %i  %i\n", sM, sIx, sIy); */
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[1][g-1][h-1]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[1][g-1][h-1]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[1][g-1][h-1]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

long scoreIy(int a, int b, int c, int g, int h){

  long sM, sIx, sIy;

  sM = aln[c].SCmatrix[0][g-1][h] + gapOPEN * factors.gap_open;
  sIx = aln[c].SCmatrix[1][g-1][h] + gapOPEN * factors.gap_open;
  sIy = aln[c].SCmatrix[2][g-1][h] + gapEXT * factors.gap_ext;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[2][g-1][h-1]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[2][g-1][h-1]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[2][g-1][h-1]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

long MscoreM(struct ALNdata tempaln[2], int c, int g, int h, short use_factors){
  
  int x=0, y=0, z=0, i=0, k=0, m=0, rest;
  float aa_orf=1, ns_code=1;
  short ana_codiert=0;  
  long s=0, as=0, ss=0, sM=0, sIx=0, sIy=0;

  g=g-1;
  h=h-1;
  
  while(tempaln[0].alignment[x]){
    while(tempaln[1].alignment[y]){
      
      if(isalpha(tempaln[0].alignment[x][h].nucl) && isalpha(tempaln[1].alignment[y][g].nucl)){
	ss = (naSCORE[tempaln[0].alignment[x][h].nuclNo][tempaln[1].alignment[y][g].nuclNo]);
	
	if(use_factors){
	  if(tempaln[0].alignment[x][h].nucl == tempaln[1].alignment[y][g].nucl){
	    ss = ss * factors.match;
	  }
	  else   ss = ss * factors.mismatch;
	}
	
	for(i=0; i<3; i++){
	  if((tempaln[0].alignment[x][h].aa[i]) &&
	     (tempaln[1].alignment[y][g].aa[i])){
	    if((isalpha(tempaln[0].alignment[x][h].aa[i])) &&
	       (isalpha(tempaln[1].alignment[y][g].aa[i]))){
	      k=0;   m=0;
	      if(i==0){ aa_orf = factors.aa_orf1;  ns_code = factors.eins_coding;}
	      if(i==1){ aa_orf = factors.aa_orf2;  ns_code = factors.zwei_coding;}
	      if(i==2){ aa_orf = factors.aa_orf3;  ns_code = factors.drei_coding;}
	      while((k<23) && (tempaln[0].alignment[x][h].aa[i] != AAorder[k]))   k++;  
	      while((m<23) && (tempaln[1].alignment[y][g].aa[i] != AAorder[m]))   m++;
	      if(k>22 || m>22) continue;
	      as = as + ((BLOSUM62[m][k] / 2) * aa_orf);
	      ss = ss * ns_code;
	    }
	  }
	}
	if( (tempaln[0].alignment[x][h].aa[0]) || (tempaln[1].alignment[y][g].aa[0]) ||
	    (tempaln[0].alignment[x][h].aa[1]) || (tempaln[1].alignment[y][g].aa[1]) ||
	    (tempaln[0].alignment[x][h].aa[2]) || (tempaln[1].alignment[y][g].aa[2]) ){
	  ana_codiert = 1;
	}
      }
      y++;
      z++;
      if(!ana_codiert){   ns_code = factors.non_code;   ss = ss * ns_code; }
      s = s + ss + as;
      ss=0;  as=0;
    }
    x++;
    y=0;
  }
  if(z){
    s = s*10;
    s = s/z;
    rest = s%10;
    s = s/10;
    if(rest >= 5)  s++;
  }
  sM = aln[c].SCmatrix[0][g][h] + s;
  sIx = aln[c].SCmatrix[1][g][h] + s;
  sIy = aln[c].SCmatrix[2][g][h] + s;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[0][g][h]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[0][g][h]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[0][g][h]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

long MscoreIx(struct ALNdata tempaln[2], int c, int g, int h){

  long sM, sIx, sIy;

  sM = aln[c].SCmatrix[0][g][h-1] + gapOPEN * factors.gap_open;
  sIx = aln[c].SCmatrix[1][g][h-1] + gapEXT * factors.gap_ext;
  sIy = aln[c].SCmatrix[2][g][h-1] + gapOPEN * factors.gap_open;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[1][g-1][h-1]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[1][g-1][h-1]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[1][g-1][h-1]='3';
    return(sIy);
  }
  return(0);
}

/* ----------------------------------------------------------------------- */

long MscoreIy(struct ALNdata tempaln[2], int c, int g, int h){

  long sM, sIx, sIy;
  
  sM = aln[c].SCmatrix[0][g-1][h] + gapOPEN * factors.gap_open;
  sIx = aln[c].SCmatrix[1][g-1][h] + gapOPEN * factors.gap_open;
  sIy = aln[c].SCmatrix[2][g-1][h] + gapEXT * factors.gap_ext;
  
  if((sM >= sIx) && (sM >= sIy)){
    aln[c].TRmatrix[2][g-1][h-1]='1';
    return(sM);
  }
  if((sIx > sM) && (sIx >= sIy)){
    aln[c].TRmatrix[2][g-1][h-1]='2';
    return(sIx);
  }
  if((sIy > sM) && (sIy > sIx)){
    aln[c].TRmatrix[2][g-1][h-1]='3';
    return(sIy);
  }
  return(0);
}

